﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Globalization;
using System.Windows.Media;

namespace DCT1D 
{ public partial class MainPage:UserControl 
  { const int N = 12;
    float[] xx0    = new float  [N]; //input
    float[] XX     = new float  [N]; //frequency amplitudes
    float[] xx1    = new float  [N]; //output
    Slider [] s    = new Slider [N];
    TextBox[] inpt = new TextBox[N];
    TextBox[] XXt  = new TextBox[N]; 
    TextBox[] oupt = new TextBox[N];
    Random random  = new Random();
    CultureInfo international = new CultureInfo("en-US");
      
    public MainPage() 
    { InitializeComponent();
      SliderArray ( s,   5 );
      TextBoxArray( inpt, 110 );
      TextBoxArray( XXt , 130 );
      TextBoxArray( oupt, 150 );
      for ( int i=0; i < N; i++ ) s[i].Value = xx0[i] = random.Next( 100 );
    }
    private void SliderArray( Slider[] s, int top )
    { Brush brush = new SolidColorBrush( Colors.LightGray );
      for ( int i=0; i < s.Length; i++ )
      { s[i] = new Slider();
        s[i].Background = brush;
        s[i].Orientation = Orientation.Vertical;
        s[i].Width  =  20;
        s[i].Height = 100;
        s[i].SetValue( Canvas.LeftProperty, (double)(105+i*32) );
        s[i].SetValue( Canvas.TopProperty,  (double)top );
        s[i].Maximum = 100;
        s[i].ValueChanged += Slider_ValueChanged;
        canvas.Children.Add( s[i] );
      }
    }
    private void TextBoxArray( TextBox[] tb, int top )
    { Thickness margin = new Thickness( 1,0,1,0 );
      for ( int i=0; i < tb.Length; i++ )
      { tb[i] = new TextBox();
        tb[i].Width  = 30;
        tb[i].Height = 20;
        tb[i].Margin = margin;
        tb[i].IsReadOnly = true;
        tb[i].FontSize = 7;
        tb[i].TextAlignment = TextAlignment.Center;
        tb[i].SetValue( Canvas.LeftProperty, (double)(100+i*32) );
        tb[i].SetValue( Canvas.TopProperty,  (double)top        );
        canvas.Children.Add( tb[i] );
      }
    }
    private void Slider_ValueChanged(object sender,RoutedPropertyChangedEventArgs<double> e) 
    { for ( int i=0; i < N; i++ )
      { if ( s[i] != (Slider)sender ) continue;
        xx0[i] = Convert.ToInt32( s[i].Value );
        inpt[i].Text = xx0[i].ToString();
        break;
      }
      DCT( xx0, XX );
      for ( int i=0; i < N; i++ )
      { XXt[i].Text = XX[i].ToString( "0.00", international );
      }
      DCTBack( XX, xx1, N );
      for ( int i=0; i < N; i++ ) oupt[i].Text = xx1[i].ToString( "0.00", international );
    }

    private void DCT( float[] xx, float[] XX )
    { int k, n, N = xx.Length;
      float sum;
      double piN =   Math.PI / N;
      float scaleFactor = 2f / N;
      for ( k=0; k < N; k++ )
      { sum = 0;
        for ( n=0; n < N; n++ ) sum += xx[n] * (float)Math.Cos( piN*(n+0.5)*k );
        XX[k] = sum*scaleFactor;
      }
    }
    private void DCTBack( float[] XX, float[] xx, int lastFrequency )
    { int k, n;
      double piN = Math.PI / lastFrequency;
      for ( k = 0; k < N; k++ )
      { float sum = XX[0] / 2;
        for ( n = 1; n < lastFrequency; n++ ) sum += XX[n] * (float)Math.Cos( piN*n*(k+0.5) );
        xx[k] = sum;
      }
    }
  }
}
